const DRAG_DELTA = 6;
const REPOSITION_DELTA = 100;

class DeviceInput {
  constructor() {
    this.mouseCoordinate = new Coordinate(0, 0);
    this.previousCoordinate = new Coordinate(0, 0);
    this.down = false;
    this.dragging = false;
    this.selected = -1;
    this.previousSelected = -1;
    this.states = document.getElementsByName('states');
    this.allSelected=[];
  }

  mouseMove(event, canvas, graph) {
    //If state is Edges
    if (this.states[4].checked === true || !this.down) return;
    //If the state is not Edges
    this.updateMouseCoordinate(event, canvas);
    if (~this.selected 
     && (this.dragging || this.previousCoordinate.distance(this.mouseCoordinate) > DRAG_DELTA)) {
      this.dragging = true;
      //If state is Origin
      if (this.states[0].checked === true)
        graph.moveOrigin(this.mouseCoordinate, fragSize);
      //If state is Axes
      else if (this.states[1].checked === true)
        graph.moveAxis(this.selected, this.mouseCoordinate, fragSize);
      //If state is Vertices
      else if (this.states[3].checked === true)
        graph.moveVertexOrbit(this.selected, this.mouseCoordinate);
      graph.selectedVID = this.selected;
      drawGraph(canvas, graph, settings);
      printGraph(graph);
      
    } 
  }

  mouseDown(event, canvas, graph) {
    this.down = true;
    this.updateMouseCoordinate(event, canvas);
    this.previousCoordinate.copy(this.mouseCoordinate);
    //If state is Origin
    if (this.states[0].checked === true && graph.origin != null)
      this.selected = -2;
    //If state is Axes
    else if (this.states[1].checked === true)
      this.selected = closestAxis(graph, this.mouseCoordinate);
    //If state is Vertices or Edges
    else if (this.states[3].checked === true || this.states[4].checked === true || this.states[5].checked === true)
      this.selected = closestVertex(graph, this.mouseCoordinate);
    graph.selectedVID = this.selected;
    drawGraph(canvas, graph, settings); 
  }

  mouseUp(event, canvas, graph) {
    this.updateMouseCoordinate(event, canvas);
    //SELECT: Origin
    if (this.states[0].checked === true) {
      this.down = false;
      this.dragging = false;
      if (graph.origin === null) {
        graph.createOrigin(this.mouseCoordinate);
      }
      else if (graph.origin.distance(this.mouseCoordinate) > REPOSITION_DELTA)
        graph.origin.copy(this.mouseCoordinate);
        if (graph.axes.length == 2)
          graph.computeLattice(fragSize);
    }
    //SELECT: Axes
    else if (this.states[1].checked === true) {
      this.down = false;
      this.dragging = false;
      if (~this.selected) {
        this.selected = -1;
      }
      else graph.createAxis(this.mouseCoordinate);
    }
    //SELECT: Fragment Size
    else if (this.states[2].checked === true) {
    }
    //SELECT: Vertices
    else if (this.states[3].checked === true) {
      this.down = false;
      this.dragging = false;
      if (~this.selected) {
        //Ctrl key || Cmd Key pressed
        if (event.ctrlKey || event.metaKey) {
            graph.deleteVertexOrbit(this.selected, fragSize); 
        }
        this.selected = -1;
      }
      //No ctrl key pressed
      else graph.createVertexOrbit(this.mouseCoordinate, fragSize);
    }
    //SELECT: Edges
    else if (this.states[4].checked === true) {
      if (~this.selected) {
        //One vertex already selected
        if (~this.previousSelected) {
          //Ctrl key || Cmd Key pressed
          if (event.ctrlKey || event.metaKey)
            graph.deleteEdgeOrbit(this.previousSelected, this.selected);
          else
            graph.createEdgeOrbit(this.previousSelected, this.selected, fragSize);          
          this.previousSelected = this.selected = -1;
        }
        //No vertex already selected
        else {
            this.previousSelected = this.selected;
        } 
      }
      else this.previousSelected = -1;
    }
    //SELECT: Faces
    else if (this.states[5].checked === true) {
      if (~this.selected) {
        this.allSelected.push(this.selected);
        //Check if vertex is the same as first selected
        if (this.allSelected[0] === this.selected && this.allSelected.length >= 3) {
          graph.computeFaceCopies(this.allSelected, fragSize);  
          this.previousSelected = this.selected = -1;
          this.allSelected = [];
        }
          
      }
      else {
        this.previousSelected = this.selected = -1;
        this.allSelected = [];
      }
    }
    
    printGraph(graph);
    graph.selectedVID = this.selected;
    graph.allSelected = this.allSelected;
    stateHandler();
  }

  updateMouseCoordinate(event, canvas) {
      const boundRect = canvas.getBoundingClientRect();
      this.mouseCoordinate.x = CANVAS_TRUE_WIDTH/(canvas.offsetWidth) * (event.clientX - boundRect.left);
      this.mouseCoordinate.y = CANVAS_TRUE_HEIGHT/(canvas.offsetHeight) * (event.clientY - boundRect.top);
  } 
}