/*==================================================================================*/
/*================================ EDGES AND SHIFTS ================================*/
/*==================================================================================*/
//checks if all elements of a shift are 0
function isZeroShift(shift) {
  if (shift.x === 0 && shift.y === 0)
    return true;
  return false;
}
//computes the shift between two realized vertices of the graph
function getEdgeShift(graph, v1Id, v2Id) {
  var v1Shift = getVertexShift(graph, graph.vertices[v1Id]);
  var v2Shift = getVertexShift(graph, graph.vertices[v2Id]);
  var edgeShift = new Coordinate(v2Shift.x, v2Shift.y);
  edgeShift.subtract(v1Shift);
  
  return edgeShift;
}
//perform change of basis to find the shift of a point. the new basis is given by the axes
function getVertexShift(graph, coordinate) {
  //need to shift the origin to (0,0) before doing the change of basis. this includes shifting the original point
  var shiftedCoord = new Coordinate(coordinate.x, coordinate.y);
  shiftedCoord.subtract(graph.origin);
  var newBasis = [[graph.axes[0].x, graph.axes[0].y], [graph.axes[1].x, graph.axes[1].y]];
  var inverseNewBasis = invert2DMatrix(newBasis);
  var newX = (inverseNewBasis[0][0] * shiftedCoord.x) + (inverseNewBasis[1][0] * shiftedCoord.y);
  var newY = (inverseNewBasis[0][1] * shiftedCoord.x) + (inverseNewBasis[1][1] * shiftedCoord.y);
  
  return new Coordinate(Math.floor(newX), Math.floor(newY));
}
function shiftSumIsEqual(goalShift, shiftHead, shiftTail) { 
  var testShift = new Coordinate(shiftTail.x, shiftTail.y);
  testShift.subtract(shiftHead);
  if (testShift.x === goalShift.x && testShift.y === goalShift.y) {
    return true;
  }
  return false;
}
function shiftIsEqual(shift1, shift2) {
  if (shift1.x === shift2.x && shift1.y === shift2.y) {
    return true;
  }
  return false;
}
//Where v1 and v2 are given as coordinates
function getEdgeRep(graph, v1, v2) {
  var edgeRep = [getVOrbit(graph, v1), getVOrbit(graph, v2)];
  var shift = new Coordinate(getVertexShift(graph, v2).x, getVertexShift(graph, v2).y);
  shift.subtract(getVertexShift(graph, v1));
  return [edgeRep, shift];
}
function getEOrbit(graph, v1Id, v2Id) {
  var v1 = graph.vertices[v1Id];
  var v2 = graph.vertices[v2Id];
  var edgeRep = [graph.vorbits[v1Id], graph.vorbits[v2Id]];
  var shift = new Coordinate(getVertexShift(graph, v2).x, getVertexShift(graph, v2).y);
  shift.subtract(getVertexShift(graph, v1));
  
  for (let i = 0; i < graph.edgeReps.length; i++) {
    var currEdgeRep = graph.edgeReps[i];
    var currShift = graph.eRepShifts[i];
    if (equalEdges(edgeRep, shift, currEdgeRep, currShift))
      return i;
  }
  return -1; //failed
}
/*============================================================================*/
/*================================ TRANSLATION ===============================*/
/*============================================================================*/
function translatePoints(points, translation) {
  for (let i = 0; i < points.length; i++) {
    points[i].add(translation);
  }
}
function getTranslatedCopy(repCoord, shift, axes) {
  var newCoord = new Coordinate(axes[0].x, axes[0].y);
  newCoord.multiply(shift.x);
  var temp = new Coordinate(axes[1].x, axes[1].y);
  temp.multiply(shift.y);
  newCoord.add(temp);
  newCoord.add(repCoord);
  
  return newCoord;
}  
/*========================================================================*/
/*================================ FIND PT ===============================*/
/*========================================================================*/
function getVertexRepCoord(graph, coordinate) {
  var shift = getVertexShift(graph, coordinate);
  var coordinateRep = new Coordinate(coordinate.x, coordinate.y);
  if (shift.x === 0 && shift.y === 0) {
    return coordinateRep;
  }
  else {
    while (shift.x < 0) {
      shift.x++;
      coordinateRep.add(graph.axes[0]);
    }
    while (shift.x >= 1) {
      shift.x--;
      coordinateRep.subtract(graph.axes[0]);
    }
    while (shift.y < 0) {
      shift.y++;
      coordinateRep.add(graph.axes[1]);
    }
    while (shift.y >= 1) {
      shift.y--;
      coordinateRep.subtract(graph.axes[1]);
    }
  }
  return coordinateRep;
}
function closestVertex(graph, coordinate) {
  let minDistance = Infinity;
  let minIndex = -1;
  for (let i = 0; i < graph.vertices.length; i++) {
    let distance = graph.vertices[i].distance(coordinate);
    if (distance <= VERTEX_CLICK_RADIUS && distance <= minDistance) {
      minDistance = distance;
      minIndex = i;
    }
  }
  return minIndex;
}
function closestAxis(graph, coordinate) {
  let minDistance = Infinity;
  let minIndex = -1;
  for (let i = 0; i < graph.axes.length; i++) {
    let realAxis = new Coordinate(graph.axes[i].x, graph.axes[i].y);
    realAxis.add(graph.origin);
    let distance = realAxis.distance(coordinate);
    if ( (distance <= AXES_CLICK_RADUIS || graph.axes.length === 2)  && distance <= minDistance) {
      minDistance = distance;
      minIndex = i;
    }
  }
  return minIndex;
}
function getVOrbit(graph, coord) {
  var vfrac = cartToFracCoord(graph, coord, true);
  for (let i = 0; i < graph.vertexRepsFrac.length; i++) {
    if (vfrac.equal(graph.vertexRepsFrac[i]))
      return i;
  }
  return -1; //failed
}
/*=======================================================================*/
/*================================ MATH =================================*/
/*=======================================================================*/
//https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Math/random
function getRandom(range) {
  return Math.random() * (range[1] - range[0]) + range[0];
}
//https://stackoverflow.com/questions/19269545/how-to-get-a-number-of-random-elements-from-an-array
function getMultipleRandom(arr, n) {
    var result = new Array(n),
        len = arr.length,
        taken = new Array(len);
    if (n > len)
        throw new RangeError("getRandom: more elements taken than available");
    while (n--) {
        var x = Math.floor(Math.random() * len);
        result[n] = arr[x in taken ? taken[x] : x];
        taken[x] = --len in taken ? taken[len] : len;
    }
    return result;
}
function getRandomInt(range) {
  return Math.floor(Math.random() * (range[1] - range[0]) + range[0]);
}
function invert2DMatrix(matrix) {
  var a = matrix[0][0];
  var b = matrix[1][0];
  var c = matrix[0][1];
  var d = matrix[1][1];
  
  var det = (a * d) - (b * c);
  var inverse = [[d/det,-c/det],[-b/det, a/det]];//each array is a column
  return inverse;
}
function dist(pt1, pt2) {
  return Math.sqrt(
    (Math.pow(pt2.x - pt1.x, 2)) + 
    (Math.pow(pt2.y - pt1.y, 2)));
}
//https://stackoverflow.com/questions/17410809/how-to-calculate-rotation-in-2d-in-javascript
function rotatePt(center, pt, rotation) {
  var rotatedPt = new Coordinate(0,0);
  rotatedPt.x = (Math.cos(rotation) * (pt.x - center.x)) + (Math.sin(rotation) * (pt.y - center.y)) + center.x;
  rotatedPt.y = (Math.cos(rotation) * (pt.y - center.y)) - (Math.sin(rotation) * (pt.x - center.x)) + center.y;
  return rotatedPt;
}
function det(vector1, vector2) {
  return vector1.x * vector2.y - vector1.y * vector2.x;
}
function magnitude2D(vec) {
 return Math.sqrt(Math.pow(vec.x,2) + Math.pow(vec.y,2)); 
}
function getAngle2D(vec1, vec2) { 
  var dot = vec1.x * vec2.x + vec1.y * vec2.y;
  var denom = magnitude2D(vec1) * magnitude2D(vec2);
  return Math.acos(dot / denom);
}
function getAngle2DDegrees(vec1, vec2) { 
  var dot = vec1.x * vec2.x + vec1.y * vec2.y;
  var denom = magnitude2D(vec1) * magnitude2D(vec2);
  return Math.acos(dot / denom) * 180 / Math.PI;
}
/*========================================================================*/
/*================================ COORDS ================================*/
/*========================================================================*/
//perform change of basis to find the fractional coordinate of a vertex
//to find the fractional coordinate of a vertex orbit representative instead, set asRep to true
function cartToFracCoord(graph, coordinate, asRep = false) {
  var shiftedCoord = new Coordinate(coordinate.x, coordinate.y);
  shiftedCoord.subtract(graph.origin);
  var newBasis = [[graph.axes[0].x, graph.axes[0].y], [graph.axes[1].x, graph.axes[1].y]];
  var inverseNewBasis = invert2DMatrix(newBasis);
  var newX = (inverseNewBasis[0][0] * shiftedCoord.x) + (inverseNewBasis[1][0] * shiftedCoord.y);
  var newY = (inverseNewBasis[0][1] * shiftedCoord.x) + (inverseNewBasis[1][1] * shiftedCoord.y);
  
  if (asRep) {
    if (newX < 0)
      newX = 1 + newX - Math.trunc(newX);
    else if (newX >= 1)
      newX = newX - Math.trunc(newX);
    if (newY < 0)
      newY = 1 + newY - Math.trunc(newY);
    else if (newY >= 1)
      newY = newY - Math.trunc(newY);
  }
  return new Coordinate(newX, newY);
}
//get all vertices in fractional coordinates
//Not in use but available just in case
function cartToFracCoords(graph, cartCoords) {
  var fracCoords = [];
  for (let i = 0; i < cartCoords.length; i++) {
    fracCoords.push(cartToFracCoord(graph, cartCoords[i]));
  }
  return fracCoords;
}
function cartToRepFracCoords(graph, cartCoords) {
  var fracCoords = [];
  for (let i = 0; i < cartCoords.length; i++) {
    fracCoords.push(cartToFracCoord(graph, cartCoords[i], true));
  }
  return fracCoords;
}
//origin + u*xaxis + v*yaxis;
function fracToCartCoord(graph, coordinate) {
  var fracCoord = new Coordinate(graph.axes[0].x, graph.axes[0].y);
  fracCoord.multiply(coordinate.x);
  fracCoord.add(graph.origin);
  var temp = new Coordinate(graph.axes[1].x, graph.axes[1].y);
  temp.multiply(coordinate.y);
  fracCoord.add(temp);
  
  return fracCoord;
}
function fracToCartCoords(graph, fracCoords) {
  var cartCoords = [];
  for (let i = 0; i < fracCoords.length; i++) {
    cartCoords.push(fracToCartCoord(graph, fracCoords[i]));
  }
  return cartCoords;
}
function equalEdges(edge1, shift1, edge2, shift2) {
  if (edge1[0] === edge2[0] && edge1[1] === edge2[1] && shift1.equal(shift2)) 
    return true;
  if (edge1[0] === edge2[1] && edge1[1] === edge2[0] && shift1.x === -shift2.x && shift1.y === -shift2.y)
    return true;
  return false;
}