class Settings {
  constructor() {
    this.display = new Map([
      ["showSpan", false],
      ["showUndirected", false],
    
      ["showVertices" , true],
      ["showVUniform", false],
      ["showVLabels", true],
      ["showVIds", false],
      ["showVOrbits", true],
      ["showVShifts", false],
      
      ["vsmall", false],
      ["vmedium", true],
      ["vlarge", false],
      
      ["showEdges", true],
      ["showEUniform", false],
      ["showELabels", false],
      ["showEIds", false],
      ["showEOrbits", true],
      ["showEShifts", false],
      ["showELengths", false],
      
      ["ethin", false],
      ["enormal", true],
      ["ethick", false],
      
      ["showFaces", true],
      ["showFUniform", false],
      
      ["showOrigin", true],
      ["showAxes", true],
      ["showLattice", true],
      ["showUCell",true]
    ]);
  }

  getDisplay(id) {
    return this.display.get(id);
  }
  
  resetElement(id, value) {
    this.display.set(id, value); 
    document.getElementById(id).checked = value;
  }
  
  reset() {
    this.resetElement("showSpan", false);
    this.resetElement("showUndirected", false);
    
    this.resetElement("showVertices", true);
    this.resetElement("showVUniform", false);
    this.resetElement("showVLabels", true);
    this.resetElement("showVIds", false);
    this.resetElement("showVOrbits", true);
    this.resetElement("showVShifts", false);
    
    this.resetElement("vsmall", false);
    this.resetElement("vmedium", true);
    this.resetElement("vlarge", false);
    
    this.resetElement("showEdges", true);
    this.resetElement("showEUniform", false);
    this.resetElement("showELabels", false);
    this.resetElement("showEIds", false);
    this.resetElement("showEOrbits", true);
    this.resetElement("showEShifts", false);
    this.resetElement("showELengths", false);
      
    this.resetElement("ethin", false);
    this.resetElement("enormal", true);
    this.resetElement("ethick", false); 
    
    this.resetElement("showFaces", true);
    this.resetElement("showFUniform", false); 
 
    this.resetElement("showOrigin", true);
    this.resetElement("showAxes", true);
    this.resetElement("showLattice", true); 
    this.resetElement("showUCell", true); 
  }

  updateDisplay(canvas, graph, id) {
    
    if (id === "showVBold") {
      this.display.set("showVBold", document.getElementById("showVBold").checked);
      if (settings.getDisplay("showVBold"))
        PT_FONT = PT_FONT_BOLD;
      else
        PT_FONT = PT_FONT_NORMAL;
    }
    else if (id == "vsmall" || id === "vmedium" || id === "vlarge") {
      this.display.set("vsmall", document.getElementById("vsmall").checked);
      this.display.set("vmedium", document.getElementById("vmedium").checked);
      this.display.set("vlarge", document.getElementById("vlarge").checked);
      if (id == "vsmall") {
      VERTEX_RADIUS = VERTEX_RADIUS_S;
      VERTEX_CLICK_RADIUS = VERTEX_RADIUS + 10;
      }
      else if (id === "vmedium") {
        VERTEX_RADIUS = VERTEX_RADIUS_M;
        VERTEX_CLICK_RADIUS = VERTEX_RADIUS + 10;
      }
      else if (id === "vlarge") {
        VERTEX_RADIUS = VERTEX_RADIUS_L;
        VERTEX_CLICK_RADIUS = VERTEX_RADIUS + 10;
      }
    }
    if (id === "showEBold") {
      this.display.set("showEBold", document.getElementById("showEBold").checked);
      if (settings.getDisplay("showEBold"))
        EDGE_FONT = EDGE_FONT_BOLD;
      else
        EDGE_FONT = EDGE_FONT_NORMAL;
    }
    else if (id == "ethin" || id === "enormal" || id === "ethick") {
      this.display.set("ethin", document.getElementById("ethin").checked);
      this.display.set("enormal", document.getElementById("enormal").checked);
      this.display.set("ethick", document.getElementById("ethick").checked);
      if (id === "ethin")
        EDGE_LINE_WIDTH = EDGE_LINE_WIDTH_THIN;
      else if (id === "enormal")
        EDGE_LINE_WIDTH = EDGE_LINE_WIDTH_NORMAL;
      else if (id === "ethick")
        EDGE_LINE_WIDTH = EDGE_LINE_WIDTH_THICK;
    }
    else if (id === "showVIds" || id === "showVOrbits" || id === "showVShifts") { 
      this.display.set("showVIds", document.getElementById("showVIds").checked);
      this.display.set("showVOrbits", document.getElementById("showVOrbits").checked);
      this.display.set("showVShifts", document.getElementById("showVShifts").checked);
    }
    else if (id === "showEIds" || id === "showEOrbits" || id === "showEShifts" || id === "showELengths") { 
      this.display.set("showEIds", document.getElementById("showEIds").checked);
      this.display.set("showEOrbits", document.getElementById("showEOrbits").checked);
      this.display.set("showEShifts", document.getElementById("showEShifts").checked);
      this.display.set("showELengths", document.getElementById("showELengths").checked);
    }
    else if (id === "showVertices") {
      this.display.set("showVertices", document.getElementById("showVertices").checked);
      if (!this.getDisplay("showVertices"))//If showVertices is false, all color checkboxes for vertices should be unchecked
        for (let i = 0; i < VBOXES; i++)
          document.getElementById("vcolor" + i).checked = false;
      else 
        for (let i = 0; i < VBOXES; i++)
          document.getElementById("vcolor" + i).checked = true;      
    }
    else if (id === "showEdges") {
      this.display.set("showEdges", document.getElementById("showEdges").checked);
      if (!this.getDisplay("showEdges"))//If showEdges is false, all color checkboxes for edges should be unchecked
        for (let i = 0; i < EBOXES; i++)
          document.getElementById("ecolor" + i).checked = false;
      else 
        for (let i = 0; i < EBOXES; i++)
          document.getElementById("ecolor" + i).checked = true;      
    }
    else
      this.display.set(id, document.getElementById(id).checked);

    //drawGraph(canvas, graph, this, deviceInput.selected);
    //drawGraph(transfCanvas, transfGraph, this, deviceInput.selected);
    drawGraph(canvas, graph, this);
    drawGraph(transfCanvas, transfGraph, this);
  }
    
  updateAll() {
    for (const id of this.display.keys()) {
      //console.log("id: " + id);
      this.display.set(id, document.getElementById(id).checked);
    }
  }
}